// FileMapper.cpp: implementation of the FileMapper class.
//
//////////////////////////////////////////////////////////////////////

#include "FileMapper.h"
#include <iostream>
#if defined(WIN32)
#else
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#endif // defined(WIN32)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FileMapper::FileMapper()
: m_mapped(0)
{

}

FileMapper::~FileMapper()
{
	close();
}

#if defined(WIN32)
bool FileMapper::open( const char* file_name, size_t length, size_t start)
{
	m_start=start;
	m_length=length;
	m_hFile=CreateFile( file_name, 
                GENERIC_READ,              // open for reading 
                FILE_SHARE_READ,           // share for reading 
                NULL,                      // no security 
                OPEN_EXISTING,             // existing file only 
                FILE_ATTRIBUTE_NORMAL,     // normal file 
                NULL);                     // no attr. template
	if ( m_hFile==INVALID_HANDLE_VALUE)
		return false;
	m_hMapping=CreateFileMapping( m_hFile, NULL, PAGE_READONLY, 0, m_length, NULL);
	if ( m_hMapping==INVALID_HANDLE_VALUE)
	{
		CloseHandle( m_hFile);
		return false;
	}
	m_mapped=MapViewOfFile( m_hMapping, FILE_MAP_READ, 0, m_start, m_length);
	if ( ! m_mapped)
		return false;

	return true;
}

void FileMapper::close()
{
	if ( m_mapped)
	{
		UnmapViewOfFile( m_mapped);
		CloseHandle( m_hMapping);
		CloseHandle( m_hFile);
	}
}
#else
bool FileMapper::open( const char* file_name, size_t length, size_t start)
{
	m_start=start;
	m_length=length;
cerr<<"Before open\n";
    m_fd=::open( file_name, O_RDONLY);
    if ( m_fd<0)
{
cerr<<"Open failed "<<file_name<<endl;
        return false;
}
    m_mapped=mmap( static_cast<void*>(0), 129761280, PROT_READ, MAP_PRIVATE, m_fd, 0);
    if ( m_mapped==MAP_FAILED)
{
cerr<<"Map failed\n";
		m_mapped=0;
		::close( m_fd);
        return false;
}

	return true;
}

void FileMapper::close()
{
	if ( m_mapped)
	{
		munmap( m_mapped, 129761280);
		m_mapped=0;
		::close( m_fd);
	}
}
#endif
